/*
 * Project name:
     DAC01 (Driving external Digital-to-Analog converter from mikroC)
 * Copyright:
     (c) MikroElektronika, 2005.
 * Description:
      This is a sample program which demonstrates the use of the Microchip's
      MCP4921 12-bit D/A converter with PIC mcu's. This device accepts digital
      input (number from 0..4095) and transforms it to the output voltage,
      ranging from 0..Vref. In this example the D/A is connected to PORTC and
      communicates with PIC through the SPI. The reference voltage on the
      mikroElektronika's DAC module is 5V. On reset the DAC is being set to the
      mid-range (~2.5V).
 * Test configuration:
     MCU:             PIC16F877A
     Dev.Board:       EasyPIC2
     Oscillator:      HS, 8.000 MHz
     Ext. Modules:    DAC
     SW:              mikroC v2.0.0.2
 * NOTES:
     - In this example, the entire DAC's resolution range (12bit -> 4096
       increments) is covered, meaning you'll need to hold a button for about
       7 minutes to get from mid-range to the end-of-range.
*/
const char _CHIP_SELECT = 1;
const char _TRUE = 0xFF;

unsigned int value;

void Init() {
  TRISB = 0;
  asm {
    BSF  TRISB,0
    BSF  TRISB,1
  }
 SPI_init();
 TRISC &= ~(_CHIP_SELECT);            // ClearBit(TRISC,CHIP_SELECT);
}//~

// DAC increments (0..4095) --> output voltage (0..Vref)
void DAC_Output(unsigned int valueDAC) {
char temp;
  PORTC &= ~(_CHIP_SELECT);           // ClearBit(PORTC,CHIP_SELECT);
                                      // Prepare for data transfer
  temp = (valueDAC >> 8) & 0x0F;      // Prepare hi-byte for transfer
  temp |= 0x30;                       // It's a 12-bit number, so only
  SPI_write(temp);                    //   lower nibble of high byte is used
  temp = valueDAC;                    // Prepare lo-byte for transfer
  SPI_write(temp);
  
  PORTC |= _CHIP_SELECT;              // SetBit(PORTC,CHIP_SELECT);
}//~


void main() {
  Init();
  DAC_Output(2048);                       // When program starts, DAC gives
  value = 2048;                           //   the output in the mid-range

  while (1) {                             // Main loop
     if ((Button(&PORTB,0,1,1)==_TRUE)    // Test button on B0 (increment)
       && (value < 4095)) {
         value++ ;
     } else {
       if ((Button(&PORTB,1,1,1)==_TRUE)  // If RB0 is not active then test
         && (value > 0)) {                //   RB1 (decrement)
         value-- ;
       }
     }
     DAC_Output(value);                   // Perform output
     Delay_ms(100);                       // Slow down key repeat pace
  }
}//~!
